use NQPHLL;

class TinyLang::Grammar is HLL::Grammar {
    rule TOP { <EXPR> ** ';' }
    
    proto token term { <...> }
    token term:sym<call> { <ident> '(' <arglist> ')' }
    token term:sym<value> { <value> }
    
    rule arglist { <EXPR> ** ',' }
    
    proto token value { <...> }
    token value:sym<string> { <?["]> <quote_EXPR> }
    
    ## Operators
    INIT {
        # Precedence levels.
        TinyLang::Grammar.O(':prec<r=>, :assoc<left>',  '%concatenation');
    }
    token infix:sym<~> { <sym> <O('%concatenation')> }
}

class TinyLang::Actions is HLL::Actions {
    method TOP($/) {
        my $program := PAST::Stmts.new();
        for $<EXPR> {
            $program.push($_.ast);
        }
        make $program;
    }
    
    method term:sym<call>($/) {
        my $call := $<arglist>.ast;
        $call.pasttype('call');
        $call.name(~$<ident>);
        make $call;
    }
    
    method term:sym<value>($/) {
        make $<value>.ast;
    }
    
    method arglist($/) {
        my $args := PAST::Op.new();
        for $<EXPR> {
            $args.push($_.ast);
        }
        make $args;
    }
    
    method value:sym<string>($/) {
        make $<quote_EXPR>.ast;
    }
    
    method infix:sym<~>($/) {
        make PAST::Op.new( :pirop('concat Sss') );
    }
}

class TinyLang::Compiler is HLL::Compiler {
}

sub MAIN(*@ARGS) {
    # Create and configure compiler object.
    my $tlcomp := TinyLang::Compiler.new();
    $tlcomp.language('tinylang');
    $tlcomp.parsegrammar(TinyLang::Grammar);
    $tlcomp.parseactions(TinyLang::Actions);

    # Enter the compiler.
    $tlcomp.command_line(@ARGS, :encoding('utf8'));
}

# Built-in subs just come here. :-)
our sub writeline($message) {
    say($message);
}
